"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.connectTypeorm = void 0;
const typeorm_1 = require("typeorm");
const entitiesChanged = (prevEntities, newEntities) => {
    if ((prevEntities === null || prevEntities === void 0 ? void 0 : prevEntities.length) !== (newEntities === null || newEntities === void 0 ? void 0 : newEntities.length)) {
        return true;
    }
    if (prevEntities && newEntities) {
        for (let i = 0; i < prevEntities.length; i++) {
            if (prevEntities[i] !== newEntities[i]) {
                return true;
            }
        }
    }
    return false;
};
const updateConnectionEntities = async (connection, entities) => {
    if (!connection || !entitiesChanged(connection.options.entities, entities)) {
        return;
    }
    connection.options.entities = entities;
    connection.buildMetadatas();
    if (connection.options.synchronize) {
        await connection.synchronize();
    }
};
let connection = null;
const connectTypeorm = async (connectionOptions) => {
    async function _connect() {
        connection = typeorm_1.getConnection(connectionOptions.name);
        if (!connection.isConnected) {
            connection = await connection.connect();
        }
    }
    if (!connection) {
        try {
            connection = await typeorm_1.createConnection(connectionOptions);
        }
        catch (error) {
            if (error.name === 'AlreadyHasActiveConnectionError') {
                await _connect();
            }
            else {
                console.error('TYPEORM_CONNECTION_ERROR', error);
            }
        }
    }
    else {
        await _connect();
    }
    if (process.env.NODE_ENV !== 'production') {
        await updateConnectionEntities(connection, connectionOptions.entities);
    }
};
exports.connectTypeorm = connectTypeorm;
